package eu.clarin.cmdi.mdservice.action;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.HashMap;
import java.io.ByteArrayInputStream;

import javax.xml.transform.TransformerException;

import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import eu.clarin.cmdi.mdservice.model.Query;
import eu.clarin.cmdi.mdservice.model.Termset;

/**
 * specialized subclass of main controller 
 * provides "Terms" the magic mix from mdrepo, compreg and datcats   
 * by dispatching the requests to appropriate internal methods and filling the inputStream with the result
 *  
 * @author vronk
 *
 */
public class TermsProxyAction extends GenericProxyAction {

	private static final long serialVersionUID = 1L;

	public static String ALL = "all";
	
	private String proxy_key = "terms";
	
	@Override
	public String getProxyKey() {
		return proxy_key;
	}
	
/* 	@Override
	public String getFullFormat() {		
		return "terms2extract";
	}
	*/
	
	/*
	 @Override
	 public String getRequestKey() {
			String key="";
			if (getActionkey()!=null) {
				key += getActionkey() + "//-" ;
			}
			if (getQ()!=null) {
				key += getQ() + "//-" ;
			} else {
				key +="//-" ;
			}
			if (getCollection()!=null) {
				key += getCollection() + "//-";
			} else {
				key +="//-" ;
			}
			if (getSquery()!=null) {
				key += getSquery() + "//-" ;
			} else {
				key +="//-" ;
			}
			if (getStartRecord()!=null) {
				key += getStartRecord() + "//-";
			}
			if (getMaximumRecords()!=null) {
				key += getMaximumRecords();
			}
			 
			return key;
	}
	 */
	public Termset getTermset() {		
		String ts_id="";
		if (getQ()!=null && !getQ().equals("")) ts_id = getQ();
	
		Termset res = (Termset) Termset.getTermset(ts_id);
		
		return res;
	}
	
	public String getTSAttr(String key) {	
		Termset ts = getTermset();
		String val=""; 
		if (ts!=null) val= ts.getAttr(key);
		 return val;
	}
	
	public URL getBaseURL() throws MalformedURLException {
			
		if (base_url == null) {
			base_url = new URL(getBaseURI());
		}
		
		return base_url;
	}
	
	
	 /**
	 * uses base_url + url_pattern (parametrized by actionkey) to form a url
	 * @return the URL to the terms
	 * @throws Exception 
	 */
	public URL getTargetRequest() throws IOException {
        // URL targetURL =new URL( base_url, compname + ".xml");
	
		
		URL targetURL = null;
		//targetURL = new URL(getBaseURI() + getQ());
		Admin.notifyUser("TPA.q: " + getBaseURI() + getQ() + getRepository());		
		targetURL = new URL(getBaseURI() + getQ() + "&maxdepth=" + getMaxdepth() + "&repository=" + getRepository());
		// TPA.targetURL:http://localhost:8080/MDService2/model/matrix/?q=all
	    return targetURL;
		
	}
	
	public InputStream getSourceStream() throws IOException{
		String type = getTSAttr("type");
		if (type.equals("dcr") | type.equals("rr")) {	
			// if dcr|rr call model/matrix/all: http://localhost:8080/MDService2/model/matrix/?q=all
			// + run through terms2extract transform (parametrized with getQ()-param!!), to serve the subset = appropriate Termset
			URL matrix_all_URL = new URL(getBaseURI() + ALL + "&repository="+ getRepository());
			//URL matrix_all_URL = new URL(getBaseURI() + ALL );
			Admin.notifyUser("TPA.getSourceStream: " + getBaseURI() + ALL);
			InputStream stream_matrix_all = matrix_all_URL.openStream(); 
			InputStream resultStream=null;
			try {
				MDTransformer transformer = new MDTransformer();
				transformer.setParams(createTransformerParams());
				resultStream = transformer.transformXML(stream_matrix_all);//, "terms2extract", getColumns(), getStartRecord(), getMaximumRecords(),getLang(),getQ(), getRepositoryPath());
			} catch (InterruptedException e) {				
				Admin.notifyUser("TPA.getSourceStream(): "+ e.getMessage());
			} catch (TransformerException e) {				
				Admin.notifyUser("TPA.getSourceStream(): "+ e.getMessage());
			}			
			return resultStream;
			
		} else {
			return getTargetRequest().openStream();
		}		  
	}
	
}